#!/usr/bin/env python

DTD_VERSION = "1.3"
LIB_VERSION = 8
__author__ = "Jerome Leclanche"
__email__ = "jerome@hearthsim.net"
__version__ = "%s.%s" % (DTD_VERSION, LIB_VERSION)


SYSTEM_DTD = "https://hearthsim.info/hsreplay/dtd/hsreplay-%s.dtd" % (DTD_VERSION)
